#!/bin/bash

test_no=0

#
# $1 - start string
# $2 - end string
# $3 - extra string
function pretty_print
{
	start_string="$1"
	end_string="$2"
	extra_string="$3"
	dot_string=""

	n_dots=$((72 - ${#start_string} - ${#end_string}))

	for ((local_i = 0; local_i < $n_dots; local_i++)); do
		dot_string="${dot_string}."
	done

	echo "$start_string$dot_string$end_string"
	if ! [ -z "$extra_string" ]; then
		echo -e "  -- $extra_string"
	fi
}

#
# $1 - string
# $2 - command
function verbose_comm
{
	exit_flag=0
	start_string="$1"
	shift

	eval $@ &> /dev/null; ret=$?;
	if ! [ "$ret" = "0" ]; then
		end_string="failed"
		exit_flag=1
	else
		end_string="ok"
	fi

	pretty_print "$start_string" "$end_string" ""

	if [ $exit_flag -eq 1 ]; then
		exit
	fi

	return $ret;
}


#
# $1 - string
# $2 - command
function check_true
{
	exit_flag=0
	test_no=$[$test_no+1]
	start_string="test $test_no: $1"
	shift

	eval $@ &> /dev/null; ret=$?;
	if ! [ "$ret" = "0" ]; then
		end_string="failed"
		extra_string="$@"
		exit_flag=1
	else
		extra_string=""
		end_string="ok"
	fi

	pretty_print "$start_string" "$end_string" "$extra_string"

	if [ $exit_flag -eq 1 ]; then
		exit
	fi

	return $ret;
}

#
# $1 - string
# $2 - command
function check_false
{
	exit_flag=0
	test_no=$[$test_no+1]
	start_string="test $test_no: $1"
	shift

	eval $@ &> /dev/null; ret=$?;
	if [ "$ret" = "0" ]; then
		end_string="failed"
		extra_string="$@"
		exit_flag=1
	else
		end_string="ok"
		extra_string=""
	fi

	pretty_print "$start_string" "$end_string" "$extra_string"

	if [ $exit_flag -eq 1 ]; then
		exit
	fi

	return $ret;
}


function get_file_attrs()
{
	rights=0`stat -c %a $1`
	uid=`stat -c %u $1`
	gid=`stat -c %g $1`
	size=`stat -c %s $1`
	case "`stat -c %F $1`" in
		"regular file") type="-f" ;;
		"regular empty file") type="-f" ;;
		"directory") type="-d" ;;
		*) type="-bad" ;;
	esac	
}

# $1 file
# $2 size
# $3 rights
# $4 uid
# $5 gid
# $6 type
# $7 path to gold file
function check_file
{
	get_file_attrs /tmp/pitix.mnt/$1

	check_true "check $1" [ $6 /tmp/pitix.mnt/$1 ]
	check_true "check $1 size" [ "$size" = "$2" ]
	check_true "check $1 rights" let \"$rights == $3\"
	check_true "check $1 uid" let \"$uid == $4\"
	check_true "check $1 gid" let \"$gid == $5\"
	check_true "check $1 data" diff /tmp/pitix.mnt/$1 $7/$1 
}

function get_statfs()
{
	cfblocks=`stat -f -c %f $1`
	cfinodes=`stat -f -c %d $1`
	cblocks=`stat -f -c %b $1`
	cinodes=`stat -f -c %c $1`
}

function check_statfs
{
	get_statfs /tmp/pitix.mnt/
	check_true "check free blocks" [ "$cfblocks" = "$fblocks" ] || echo "$cfblocks" = "$fblocks" 
	check_true "check free inodes" [ "$cfinodes" = "$finodes" ] || echo "$cfinodes" = "$finodes" 
}


# $1 file
# $2 size
# $3 rights
# $4 uid
# $5 gid
function do_create
{
	verbose_comm "generating random data" head -c$2 /dev/urandom \> /tmp/pitix.rw/$1
	chmod $3 /tmp/pitix.rw/$1
	chown $4.$5 /tmp/pitix.rw/$1
	check_true "copy file to /tmp/pitix.mnt/$1" \ 
	cp -p /tmp/pitix.rw/$1 /tmp/pitix.mnt/$1 && \
	finodes=$[$finodes-1] && \
	fblocks=$[$fblocks-($2+$block_size-1)/$block_size]
	check_file $1 $2 $3 $4 $5 -f /tmp/pitix.rw/
	check_statfs 
}

# $1 dir
# $2 rights
# $3 uid
# $4 gid
function do_mkdir
{
	mkdir /tmp/pitix.rw/$1
	chmod $2 /tmp/pitix.rw/$1 
	chown $3.$4 /tmp/pitix.rw/$1

	check_true "check mkdir" mkdir /tmp/pitix.mnt/$1 && \
	finodes=$[$finodes-1] && \
	fblocks=$[$fblocks-1]

	check_true "check chmod" chmod $2 /tmp/pitix.mnt/$1
	check_true "check chown" chown $3.$4 /tmp/pitix.mnt/$1

	check_file $1 $block_size $2 $3 $4 -d /tmp/pitix.rw/
	check_statfs
}

# $1 file
function do_unlink
{
	get_file_attrs /tmp/pitix.mnt/$1

	rm -f /tmp/pitix.rw/$1
	check_true "check unlink 1" rm /tmp/pitix.mnt/$1
	check_true  "check unlink 2" ! [ -e /tmp/pitix.mnt/$1 ] && \
	finodes=$[$finodes+1] && \
	fblocks=$[$fblocks+($size+$block_size-1)/$block_size]
	check_statfs  
}

# $1 file
function do_false_unlink
{
	check_false "check false unlink " rm /tmp/pitix.mnt/$1
}


# $1 file
function do_rmdir
{
	rmdir /tmp/pitix.rw/$1
	check_true "check rmdir 1" rmdir /tmp/pitix.mnt/$1
	check_true  "check rmdir 2" ! [ -e /tmp/pitix.mnt/$1 ]
	finodes=$[$finodes+1];
	fblocks=$[$fblocks+1]
	check_statfs 
}

# $1 file
function do_false_rmdir
{
	check_false "check false rmdir" rmdir /tmp/pitix.mnt/$1
}


# $1 file
function do_truncate
{
	get_file_attrs /tmp/pitix.mnt/$1
	old_blocks=$[($size+$block_size-1)/$block_size]

	echo dummy > /tmp/pitix.rw/$1
	check_true "check truncate" echo dummy \> /tmp/pitix.mnt/$1 && fblocks=$[$fblocks+$old_blocks-1]

	check_file $1 6 $rights $uid $gid -f /tmp/pitix.rw/
	check_statfs
}


# $1 block size
# $2 inode_data_blocks
function test_rw
{
	block_size=$1
	idata_blocks=$2
	de_per_dir=$[$block_size/20]
	blocks=$[8*$block_size]
	inodes=$[32*$block_size/(20+2*idata_blocks)]
	fblocks=$[$blocks-1]
	finodes=$[$inodes-1]

	rm -Rf /tmp/pitix.loop /tmp/pitix.rw /tmp/pitix.mnt &> /dev/null; mkdir /tmp/pitix.rw; mkdir /tmp/pitix.mnt

	head -c4096 /dev/zero > /tmp/pitix.loop
	check_false "mounting bad fs 1" mount -t pitix /tmp/pitix.loop /tmp/pitix.mnt/ -o loop
	echo > /tmp/pitix.loop
	check_false "mounting bad fs 2" mount -t pitix /tmp/pitix.loop /tmp/pitix.mnt/ -o loop

	verbose_comm "formatting fs" ./mk_pitix $1 $2 /tmp/pitix.loop 
	check_true "mounting fs" mount -t pitix /tmp/pitix.loop /tmp/pitix.mnt/ -o loop

	get_statfs /tmp/pitix.mnt
	check_true "check total blocks" [ "$cblocks" = "$blocks" ] || echo "$cblocks" = "$blocks" 
	check_true "check total inodes" [ "$cinodes" = "$inodes" ] || echo "$cinodes" = "$inodes"

	do_mkdir lots_of_files 0777 0 0
	for((i=0; i<$de_per_dir; i++)); do
		do_create lots_of_files/file$i $[$i*1024] 0$[$i%8]$[($i/2)%8]$[($i/4)%8] $[$i%100] $[$i*32%100]
	done

	check_false "dir overflow" touch /tmp/pitix.mnt/lots_of_files/the_drop

	check_statfs

	check_true "umounting fs" umount /tmp/pitix.mnt/
	check_true "mounting fs" mount -t pitix /tmp/pitix.loop /tmp/pitix.mnt/ -o loop

	check_statfs

	for((i=0; i<$de_per_dir; i++)); do
		check_file lots_of_files/file$i $[$i*1024] 0$[$i%8]$[($i/2)%8]$[($i/4)%8] $[$i%100] $[$i*32%100] -f /tmp/pitix.rw/
		if let "$[$i%2] == 0"; then
			do_truncate lots_of_files/file$i
		else
			do_unlink lots_of_files/file$i
			do_false_unlink lots_of_files/file$i
		fi
	done

	for((i=1; i<$de_per_dir; i++)); do
		do_mkdir dir$i 0666 0 0
		do_create dir$i/dummy 4096 0666 0 0  
	done

	for((i=1; i<$de_per_dir; i++)); do
		do_false_rmdir dir$i
		do_unlink dir$i/dummy
		do_rmdir dir$i
	done


	check_false "module unloading" rmmod pitix
	check_true "umounting fs" umount /tmp/pitix.mnt/
}

function test_ro()
{
	rm -Rf /tmp/pitix.loop /tmp/pitix.mnt /tmp/pitix.ro &> /dev/null; mkdir /tmp/pitix.ro; mkdir /tmp/pitix.mnt
	gunzip pitix.loop.gz && cp pitix.loop /tmp/pitix.loop
	tar xfz pitix.files.tar.gz -C /tmp/pitix.ro         

	check_true "mounting fs" mount -t pitix /tmp/pitix.loop /tmp/pitix.mnt/ -o loop

	for file in `find /tmp/pitix.ro -type f | cut -f4- -d/`; do
		get_file_attrs /tmp/pitix.ro/$file
		check_file $file $size $rights $uid $gid $type /tmp/pitix.ro/
	done

	check_false "module unloading" rmmod pitix
	check_true "umounting fs" umount /tmp/pitix.mnt/
}

check_true  "module loading" insmod ../pitix.ko
check_true  "test /proc/filesystems" grep pitix /proc/filesystems
test_ro
check_true "module unloading" rmmod pitix

check_true  "module loading" insmod ../pitix.ko
check_true  "test /proc/filesystems" grep pitix /proc/filesystems
test_rw 512 64
check_true "module unloading" rmmod pitix


# vim: set tabstop=4 shiftwidth=4:
